@@ -287,4 +287,29 @@ arma::mat invWbyRcpp(double sigsq, arma::mat A) {
   return arma::mat (as_scalar(1/sigsq)*(Ds - as_scalar(1/sigsq)*A*Q*trans(A)));
 }
 
+const double log2pi = std::log(2.0 * M_PI);
+
+// [[Rcpp::depends("RcppArmadillo")]]
+// [[Rcpp::export]]
+arma::vec dmvnormRcpp(arma::mat x,  
+                      arma::mat mean,  
+                      arma::mat sigma, 
+                      bool logd = false) { 
+  int n = x.n_rows;
+  int xdim = x.n_cols;
+  arma::vec out(n);
+  arma::mat rooti = arma::trans(arma::inv(trimatu(arma::chol(sigma))));
+  double rootisum = arma::sum(log(rooti.diag()));
+  double constants = -(static_cast<double>(xdim)/2.0) * log2pi;
+  
+  for (int i=0; i < n; i++) {
+    arma::vec z = rooti * arma::trans( x.row(i) - mean.row(i));    
+    out(i)      = constants - 0.5 * arma::sum(z%z) + rootisum;     
+  }  
+  
+  if (logd == false) {
+    out = exp(out);
+  }
+  return(out);
+}
 
